#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Initialization module for artella
"""

from __future__ import print_function, division, absolute_import

from artella.core.dcc.app import *
from artella.core.dcc.callback import *
from artella.core.dcc.dialog import *
from artella.core.dcc.menu import *
from artella.core.dcc.parser import *
from artella.core.dcc.progress import *
from artella.core.dcc.ui import *
from artella.core.dcc.window import *




#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains ArtellaDrive Qt related utils classes and functions
"""

from __future__ import print_function, division, absolute_import

import os

import artella.dcc as dcc
from artella import register
from artella.core import utils, qtutils

if qtutils.QT_AVAILABLE:
    from artella.externals.Qt import QtGui


class ResourcesManager(object):
    def __init__(self):
        super(ResourcesManager, self).__init__()

        self._resources_paths = list()
        self._resources_cache = {
            'icon': {}, 'pixmap': {}, 'style': {}
        }

    def register_resources_path(self, resources_path):
        """
        Registers a path where resources will be searched
        :param str resources_path: Path to search resources in
        """

        if not resources_path or not os.path.isdir(resources_path):
            return

        resources_path = utils.clean_path(resources_path)
        if resources_path in self._resources_paths:
            return

        self._resources_paths.append(resources_path)

        dcc.register_dcc_resource_path(resources_path)
        icons_path = os.path.join(resources_path, 'icons')
        if os.path.isdir(icons_path):
            dcc.register_dcc_resource_path(icons_path)

    def icon(self, name, extension='png', color=None):
        """
        Returns Artella icon
        :param name:
        :param extension:
        :param color:
        :return: QIcon
        """

        resource_type = 'icon'

        new_icon, resource_key = self._get_resource(resource_type, name=name, extension=extension, color=color)
        if not new_icon:
            return None if not qtutils.QT_AVAILABLE else QtGui.QIcon()

        if not new_icon.isNull():
            self._register_resource(resource_type, resource_key, new_icon)

        return new_icon

    def pixmap(self, name, extension='png', color=None):
        """
        Returns Artella pixmap resource
        :param name:
        :param extension:
        :param color:
        :return:
        """

        resource_type = 'pixmap'

        new_pixmap, resource_key = self._get_resource(resource_type, name=name, extension=extension, color=color)
        if not new_pixmap:
            return None if not qtutils.QT_AVAILABLE else QtGui.QPixmap()

        if not new_pixmap.isNull():
            self._register_resource(resource_type, resource_key, new_pixmap)

        return new_pixmap

    def style(self, name, extension='css'):
        """
        Returns Artella style resource
        :param name:
        :param extension:
        :return:
        """

        resource_type = 'style'

        new_style, resource_key = self._get_resource('style', name=name, extension=extension)
        if not new_style:
            return None

        self._register_resource(resource_type, resource_key, new_style)

        return new_style

    def _get_resource(self, resource_type, name, extension, **kwargs):
        """
        Internal function that returns a resource based on its type
        :param resource_type:
        :return: object
        """

        color = kwargs.get('color', '')
        file_name = '{}{}'.format(name, extension)
        file_key = '{}{}'.format(file_name.lower(), color)

        if not qtutils.QT_AVAILABLE or not self._resources_paths or resource_type not in self._resources_cache:
            return None, file_key

        if not extension.startswith('.'):
            extension = '.{}'.format(extension)

        if file_key in self._resources_cache[resource_type]:
            return self._resources_cache[resource_type][file_key], file_key

        for resource_path in self._resources_paths:
            for root, dirs, files in os.walk(resource_path):
                for path in files:
                    path_name, path_ext = os.path.splitext(path)
                    if path_name == name and path_ext == extension:
                        res_path = os.path.join(root, path)
                        new_res = getattr(self, '_{}'.format(resource_type))(res_path, color=color)
                        return new_res, file_key

        return None, file_key

    def _register_resource(self, resource_type, resource_key, resource):
        """
        Internal function used to register resources in its proper cache
        :param resource_type: str
        :param resource_key: str
        :param resource: object
        :return:
        """

        if resource_type not in self._resources_cache:
            return

        self._resources_cache[resource_type][resource_key] = resource

    def _icon(self, icon_path, color=None, **kwargs):
        """
        Internal function returns an Artella icon resource
        :param icon_path:
        :param color:
        :return: QIcon
        """

        new_icon = qtutils.icon(icon_path, color=color)

        return new_icon

    def _pixmap(self, pixmap_path, color=None, **kwargs):
        """
        Internal function returns an Artella pixmap resource
        :param file_key: str
        :param pixmap_path: str
        :param color:
        :return:
        """

        new_pixmap = qtutils.pixmap(pixmap_path, color=color)

        return new_pixmap

    def _style(self, style_path, **kwargs):
        """
        Internal function returns an Artella style resource
        :param style_path: str
        :return:
        """

        new_style = qtutils.style(style_path)

        return new_style


@utils.Singleton
class ArtellaResourcesManagerSingleton(ResourcesManager, object):
    def __init__(self):
        ResourcesManager.__init__(self)


register.register_class('ResourcesMgr', ArtellaResourcesManagerSingleton)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Logger module for Artella
"""

from __future__ import print_function, division, absolute_import

import os
import logging.config


def create_logger():

    logger_path = os.path.normpath(os.path.join(os.path.expanduser('~'), 'artella', 'logs'))
    if not os.path.isdir(logger_path):
        os.makedirs(logger_path)

    logging.config.fileConfig(
        os.path.normpath(os.path.join(os.path.dirname(__file__), 'logging.ini')), disable_existing_loggers=False)




#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains object to manage Artella widgets theme
"""

from __future__ import print_function, division, absolute_import

import artella
from artella.widgets import color


class ArtellaTheme(object):
    def __init__(self, main_color=None):
        super(ArtellaTheme, self).__init__()

        self._init_sizes()
        self._init_fonts()
        self._init_colors()
        self.set_main_color(main_color or color.ArtellaColors.DEFAULT)

        self._default_style = artella.ResourcesMgr().style('artella')
        self.unit = 'px'
        self.default_size = self.medium
        self.text_error_color = self.error_7
        self.text_color_inverse = "#fff"
        self.text_warning_color = self.warning_7

    def set_main_color(self, main_color):
        self.main_color = main_color
        self.main_color_1 = color.generate_color(main_color, 1)
        self.main_color_2 = color.generate_color(main_color, 2)
        self.main_color_3 = color.generate_color(main_color, 3)
        self.main_color_4 = color.generate_color(main_color, 4)
        self.main_color_5 = color.generate_color(main_color, 5)
        self.main_color_6 = color.generate_color(main_color, 6)
        self.main_color_7 = color.generate_color(main_color, 7)
        self.main_color_8 = color.generate_color(main_color, 8)
        self.main_color_9 = color.generate_color(main_color, 9)
        self.main_color_10 = color.generate_color(main_color, 10)
        self.item_hover_bg = self.main_color_1

    def apply(self, widget):
        widget.setStyleSheet(self._default_style.substitute(vars(self)))

    def _init_sizes(self):
        self.tiny = 18
        self.small = 24
        self.medium = 32
        self.large = 40
        self.huge = 48
        self.huge_icon = self.huge - 20
        self.large_icon = self.large - 16
        self.medium_icon = self.medium - 12
        self.small_icon = self.small - 10
        self.tiny_icon = self.tiny - 8

    def _init_fonts(self):
        self.font_family = '"Helvetica Neue",Helvetica,Arial,sans-serif'
        self.font_size_base = 14
        self.font_size_large = self.font_size_base + 2
        self.font_size_small = self.font_size_base - 2
        self.h1_size = int(self.font_size_base * 2.71)
        self.h2_size = int(self.font_size_base * 2.12)
        self.h3_size = int(self.font_size_base * 1.71)
        self.h4_size = int(self.font_size_base * 1.41)

    def _init_colors(self):
        self.artella_color = color.ArtellaColors.ARTELLA
        self.info_color = color.ArtellaColors.BLUE
        self.success_color = color.ArtellaColors.GREEN
        self.error_color = color.ArtellaColors.RED
        self.warning_color = color.ArtellaColors.YELLOW

        self.info_1 = color.fade_color(self.info_color, '15%')
        self.info_2 = color.generate_color(self.info_color, 2)
        self.info_3 = color.fade_color(self.info_color, '35%')
        self.info_4 = color.generate_color(self.info_color, 4)
        self.info_5 = color.generate_color(self.info_color, 5)
        self.info_6 = color.generate_color(self.info_color, 6)
        self.info_7 = color.generate_color(self.info_color, 7)
        self.info_8 = color.generate_color(self.info_color, 8)
        self.info_9 = color.generate_color(self.info_color, 9)
        self.info_10 = color.generate_color(self.info_color, 10)

        self.success_1 = color.fade_color(self.success_color, '15%')
        self.success_2 = color.generate_color(self.success_color, 2)
        self.success_3 = color.fade_color(self.success_color, '35%')
        self.success_4 = color.generate_color(self.success_color, 4)
        self.success_5 = color.generate_color(self.success_color, 5)
        self.success_6 = color.generate_color(self.success_color, 6)
        self.success_7 = color.generate_color(self.success_color, 7)
        self.success_8 = color.generate_color(self.success_color, 8)
        self.success_9 = color.generate_color(self.success_color, 9)
        self.success_10 = color.generate_color(self.success_color, 10)

        self.warning_1 = color.fade_color(self.warning_color, '15%')
        self.warning_2 = color.generate_color(self.warning_color, 2)
        self.warning_3 = color.fade_color(self.warning_color, '35%')
        self.warning_4 = color.generate_color(self.warning_color, 4)
        self.warning_5 = color.generate_color(self.warning_color, 5)
        self.warning_6 = color.generate_color(self.warning_color, 6)
        self.warning_7 = color.generate_color(self.warning_color, 7)
        self.warning_8 = color.generate_color(self.warning_color, 8)
        self.warning_9 = color.generate_color(self.warning_color, 9)
        self.warning_10 = color.generate_color(self.warning_color, 10)

        self.error_1 = color.fade_color(self.error_color, '15%')
        self.error_2 = color.generate_color(self.error_color, 2)
        self.error_3 = color.fade_color(self.error_color, '35%')
        self.error_4 = color.generate_color(self.error_color, 4)
        self.error_5 = color.generate_color(self.error_color, 5)
        self.error_6 = color.generate_color(self.error_color, 6)
        self.error_7 = color.generate_color(self.error_color, 7)
        self.error_8 = color.generate_color(self.error_color, 8)
        self.error_9 = color.generate_color(self.error_color, 9)
        self.error_10 = color.generate_color(self.error_color, 10)

        self.title_color = "#ffffff"
        self.main_text_color = "#d9d9d9"
        self.secondary_text_color = "#a6a6a6"
        self.disable_color = "#737373"
        self.border_color = "#1e1e1e"
        self.divider_color = "#262626"
        self.header_color = "#0a0a0a"
        self.icon_color = "#a6a6a6"
        self.background_color = "#323232"
        self.background_selected_color = "#292929"
        self.background_in_color = "#3a3a3a"
        self.background_out_color = "#494949"
        self.mask_color = color.fade_color(self.background_color, '90%')
        self.toast_color = "#555555"



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains constants definitions used by Artella Drive
"""

from __future__ import print_function, division, absolute_import

# Defines Local Environment Variable that sets Artella root folder location
# This is used as fallback when ArtellaApp/URI is not available
ALR = 'ART_LOCAL_ROOT'

# Defines Local Environment Variables used in old Artella versions
OLD_LOCAL_ROOTS = ['AM_LOCAL_ROOT', 'AM_LOCAL_STORAGE_TOP']

# Defines Local Environment Variable that sets Artella External DCCs location
# This is used to register custom DCC implementations
AED = 'ARTELLA_EXTERNAL_DCCS'

# Defines package namespace where Artella DCC plugins are located
ARTELLA_DCCS_NAMESPACE = 'artella.dccs'

# Defines URI scheme used by Artella
ARTELLA_URI_SCHEME = 'artella'

# Defines default IP host used by the client
DEFAULT_HOST = '127.0.0.1'

# Defines default port used by the client
DEFAULT_PORT = 29282

# Defines default buffer size for web socket requests to Artella Drive App
DEFAULT_BUFFER_SIZE = 2000

# Defines the file name used by Artella Drive App to store project specific metadata
PROJECT_METADATA_FILE = '.artella-folder.yaml'

# Defines authentication header
AUTH_HEADER = 'artella-file-challenge artella-challenge {}'

# Defines default note used by lock operation
DEFAULT_LOCK_NOTE = 'Lock for use by plugin'

# Control codes CL/RF to create new lines.
CRLF = "\r\n"

# Configuration file for Artella Plugins
ARTELLA_PLUGIN_CONFIG = 'artella-plugin.json'



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains DCC abstract application implementation
"""

from __future__ import print_function, division, absolute_import

from artella.core.dcc import reroute
from artella.core.utils import abstract


@reroute
@abstract
def name():
    """
    Returns name of current DCC

    :return: Returns name of DCC without any info about version
    :rtype: str
    """

    pass


@reroute
@abstract
def version():
    """
    Returns version of DCC application

    :return: Returns integer number indicating the version of the DCC application
    :rtype: int
    """

    pass


@reroute
@abstract
def extensions():
    """
    Returns a list of available extension for DCC application

    :return: List of available extensions with the following format: .{EXTENSION}
    :rtype: list(str)
    """

    pass


@reroute
@abstract
def scene_name():
    """
    Returns the name of the current scene

    :return: Full file path name of the current scene. If no file is opened, None is returned.
    :rtype: str or None
    """

    pass


@reroute
@abstract
def new_scene(force=True):
    """
    Creates a new scene inside DCC
    :param force: True to skip saving of the current opened DCC scene; False otherwise.
    :return: True if the new scene is created successfully; False otherwise.
    :rtype: bool
    """

    pass


@reroute
@abstract
def scene_is_modified():
    """
    Returns whether or not current opened DCC file has been modified by the user or not

    :return: True if current DCC file has been modified by the user; False otherwise
    :rtype: bool
    """

    pass


@reroute
@abstract
def open_scene(file_path, save=True):
    """
    Opens DCC scene file
    :param str file_path: Absolute local file path we want to open in current DCC
    :param bool save: Whether or not save current opened DCC scene file
    :return: True if the save operation was successful; False otherwise
    :rtype: bool
    """

    pass


@reroute
@abstract
def save_scene(force=True, **kwargs):
    """
    Saves DCC scene file

    :param bool force: Whether to force the saving operation or not
    :return: True if the save operation was successful; False otherwise
    :rtype: bool
    """

    pass


@reroute
@abstract
def import_scene(file_path):
    """
    Opens scene file into current opened DCC scene file
    :param str file_path: Absolute local file path we want to import into current DCC
    :return: True if the import operation was successful; False otherwise
    :rtype: bool
    """

    pass


@reroute
@abstract
def reference_scene(file_path, **kwargs):
    """
    References scene file into current opened DCC scene file
    :param str file_path: Absolute local file path we want to reference into current DCC
    :return: True if the reference operation was successful; False otherwise
    :rtype: bool
    """

    pass


@reroute
@abstract
def supports_uri_scheme():
    """
    Returns whether or not current DCC support URI scheme implementation

    :return: True if current DCC supports URI scheme implementation; False otherwise
    """

    pass


@reroute
@abstract
def pass_message_to_main_thread_fn():
    """
    Returns function used by DCC to execute a function in DCC main thread in the next idle event of that thread.

    :return If DCC API supports it, returns function to call a function in main thread from other thread
    """

    pass


@reroute
@abstract
def is_batch():
    """
    Returns whether or not current DCC is being executed in batch mode (no UI)
    :return: True if current DCC session is being executed in batch mode; False otherwise.
    :rtype: bool
    """

    pass


@reroute
@abstract
def clean_path(file_path):
    """
    Cleans given path so it can be properly used by current DCC

    :param str file_path: file path we want to clean
    :return: Cleaned version of the given file path
    :rtype: str
    """

    pass


@reroute
@abstract
def get_installation_paths(versions=None):
    """
    Returns installation path of the given versions of current DCC

    :param list(int) versions: list of versions to find installation paths of. If not given, current DCC version
        installation path will be returned
    :return: List of installation paths of the given DCC versions
    :rtype: list(str)
    """

    pass


@reroute
@abstract
def execute_deferred(fn):
    """
    Executes given function in deferred mode (once DCC UI has been loaded)
    :param callable fn: Function to execute in deferred mode
    :return:
    """

    pass


@reroute
@abstract
def register_dcc_resource_path(resources_path):
    """
    Registers path into given DCC so it can find specific resources
    :param resources_path: str, path we want DCC to register
    """

    pass


#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains dcc module initialization
"""

from __future__ import print_function, division, absolute_import

import os
import sys
import logging
from functools import wraps
from importlib import import_module

from artella.core import consts, utils

logger = logging.getLogger('artella')

# Directories where DCCs implementations are located.
DCCS_DIRS = list()

# Cached dict of available dcc. This listed is cached in import time because we assume that no new DCCs
# implementations will be added in runtime
DCCS = dict()

# Cached active DCC. We use it to avoid repetitive checks. We assume that a DCC environment will not change
# during a session.
CURRENT_DCC = None

# Cached active DCC module name.
CURRENT_DCC_MODULE = None

# Cached used to store all the reroute paths done during a session. We assume DCC reroutes will not change
# during a session.
DCC_REROUTE_CACHE = dict()


def dccs():
    """
    Returns a list with all supported DCCs

    :return: List with the names of all supported DCCs
    :rtype: list(str)
    """

    return DCCS.keys()


def current_dcc():
    """
    Returns the DCC loaded in current session for current environment
    If the current DCC is not already cached it will be automatically detected taking into account available DCC
    implementations. Otherwise, cached DCC is returned.

    :return: Name of the current used DCC
    :rtype: str
    """

    # If active DCC is already cache, we use it
    global CURRENT_DCC
    global CURRENT_DCC_MODULE
    if CURRENT_DCC:
        return CURRENT_DCC

    # If DCC is not available we make sure that we check all available dirs where DCC implementation can be located
    global DCCS_DIRS
    global DCCS
    dcc_paths_str = os.environ.get(consts.AED, '')
    if dcc_paths_str:
        dcc_paths_split = [utils.clean_path(pth) for pth in dcc_paths_str.split(';')]
        for dcc_path in dcc_paths_split:
            if not dcc_path or not os.path.isdir(dcc_path) or dcc_path in DCCS_DIRS:
                continue
            DCCS_DIRS.append(dcc_path)
    if DCCS_DIRS:
        for dcc_dir in DCCS_DIRS:
            for dcc_folder in os.listdir(dcc_dir):
                if not dcc_folder.startswith('__') and not dcc_folder.endswith('__') and dcc_folder != 'abstract':
                    DCCS[dcc_folder] = utils.clean_path(os.path.join(dcc_dir, dcc_folder))
                    if DCCS[dcc_folder] not in sys.path:
                        sys.path.append(DCCS[dcc_folder])

    # Loop through all available DCCs and check which one is available in current session
    for dcc_name, dcc_dir in DCCS.items():
        dcc_namespace_split = dcc_name.split('-')
        dcc_names = [dcc_name, dcc_namespace_split[-1]] if dcc_namespace_split else [dcc_name]
        for dcc in dcc_names:
            try:
                dcc_module_name = '{}.{}'.format(consts.ARTELLA_DCCS_NAMESPACE, dcc)
                import_module(dcc_module_name)
                CURRENT_DCC = dcc
                CURRENT_DCC_MODULE = dcc_module_name
                logger.info('Current DCC: {}'.format(CURRENT_DCC))
                return CURRENT_DCC
            except ImportError as exc:
                continue


def reroute(fn):
    """
    Decorator that reroutes the function call on runtime to the specific DCC implementation of the function
    Rerouted function calls are cached, and are only loaded once.
    The used DCC API will be retrieved from the current session, taking into account the current available
    implementations

    :param fn:
    """

    @wraps(fn)
    def wrapper(*args, **kwargs):

        global DCC_REROUTE_CACHE

        dcc = current_dcc()
        if not dcc:
            return None

        # From the current function and DCC we retrieve module path where DCC implementation should be located
        fn_split = fn.__module__.split('.')
        dcc_reroute_path = '{}.{}.{}'.format(consts.ARTELLA_DCCS_NAMESPACE, dcc, '.'.join(fn_split[3:]))
        dcc_reroute_fn_path = '{}.{}'.format(dcc_reroute_path, fn.__name__)
        if dcc_reroute_fn_path not in DCC_REROUTE_CACHE:
            try:
                dcc_reroute_module = import_module(dcc_reroute_path)
            except ImportError as exc:
                raise NotImplementedError(
                    '{} | Function {} not implemented! {}'.format(dcc, dcc_reroute_fn_path, exc))
            except Exception as exc:
                raise exc

            # Cache reroute call, next calls to that function will use cache data
            dcc_reroute_fn = getattr(dcc_reroute_module, fn.__name__)
            DCC_REROUTE_CACHE[dcc_reroute_fn_path] = dcc_reroute_fn

        return DCC_REROUTE_CACHE[dcc_reroute_fn_path](*args, **kwargs)

    return wrapper


def callbacks():
    """
    Returns a list of callbacks based on DCC available callbacks
    :return: List of available DCC callbacks
    :rtype: list
    """

    callbacks_list = list()
    for k, v in DccCallbacks.__dict__.items():
        if k.startswith('__') or k.endswith('__'):
            continue
        callbacks_list.append(v[0])

    return callbacks_list


class DccCallbacks(object):
    """
    Class that contains all callbacks that can be supported by DCCs
    """

    Shutdown = ('Shutdown', {'type': 'simple'})
    BeforeOpenCheck = ('BeforeOpenCheck', {'type': 'simple'})
    AfterOpen = ('AfterOpen', {'type': 'simple'})
    SceneBeforeSave = ('SceneBeforeSave', {'type': 'simple'})
    SceneCreated = ('SceneCreated', {'type': 'simple'})
    AfterLoadReference = ('AfterLoadReference', {'type': 'simple'})
    BeforeCreateReferenceCheck = ('BeforeCreateReferenceCheck', {'type': 'simple'})



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains ArtellaProject implementation
"""


class ArtellaProject(object):
    def __init__(self, session, project_id, name, client):
        super(ArtellaProject, self).__init__()

        self._session = session
        self._id = project_id
        self._name = name
        self._client = client

    # ==============================================================================================================
    # PROPERTIES
    # ==============================================================================================================

    @property
    def remote_session(self):
        return self._session

    @property
    def id(self):
        return self._id

    @property
    def name(self):
        return self._name

    # ==============================================================================================================
    # CREATE
    # ==============================================================================================================

    @classmethod
    def create(cls, project_id, client):
        """
        Creates a new instance of the project taking into account th e given project ID

        :param str project_id: ID of the project
        :param ArtellaDriveClient client: Artella Drive Client this project is linked to
        :return: New instance of the project with the give ID. Exception is raised if project cannot be created
        :rtype: ArtellaProject
        """

        if not client:
            raise Exception(
                'Project with ID "{}" cannot be created because not Artella Drive Client instance was given'.format(
                    project_id))
        if not project_id:
            raise Exception('Project cannot be created because no ID was given')

        projects = client.get_remote_projects()

        new_project = None
        for remote_session, project_ids in projects.items():
            if project_id in project_ids:
                project_name = client.get_project_name(project_id)
                new_project = cls(session=remote_session, project_id=project_id, name=project_name, client=client)
                break

        if not new_project:
            raise Exception('Impossible to create project with ID: "{}"'.format(project_id))

        return new_project



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains ArtellaDrive utils classes and functions
"""

from __future__ import print_function, division, absolute_import

import os
import sys
import imp
import time
import json
import shutil
import fnmatch
import inspect
import logging
import importlib
import subprocess
from functools import wraps

logger = logging.getLogger('artella')


def is_python2():
    """
    Returns whether or not current version is Python 2
    :return: bool
    """

    return sys.version_info.major == 2


def is_python3():
    """
    Returns whether or not current version is Python 3
    :return: bool
    """

    return sys.version_info.major == 3


def clean_path(path):
    """
    Returns a cleaned path to make sure that we do not have problems with path slashes
    :param str path: path we want to clean
    :return: clean path
    :rtype: str
    """

    # Convert '~' Unix char to user's home directory and remove spaces and bad slashes
    path = os.path.expanduser(str(path).decode('utf-8'))
    path = str(path.replace('\\', '/').replace('//', '/').rstrip('/').strip())

    # Fix server paths
    is_server_path = path.startswith('\\')
    while '\\' in path:
        path = path.replace('\\', '//')
    if is_server_path:
        path = '//{}'.format(path)

    # Fix web paths
    if not path.find('https://') > -1:
        path = path.replace('//', '/')

    return path


def split_path(path, clean_drive=True):
    """
    Splits given paths in all its sub parts

    :param str path: Path we want to retrieve parts of
    :param clean_drive: True if we want to remove the drive from the result; False otherwise.
    :return: List of sub parts in the given path
    :rtype: list(str)
    """

    if clean_drive:
        path = os.path.splitdrive(path)[-1]

    return [next(part for part in path.split(os.path.sep) if part)]


def clear_list(list_to_clear):
    """
    Clears given Python list. Works fine for both Python 2 and Python 3.

    :param list_to_clear: list
    """

    if is_python2():
        del list_to_clear[:]
    else:
        list_to_clear.clear()


def force_list(var, remove_duplicates=False):
    """
    Returns given variable as a list

    :param object var: variable we want to convert into a list
    :param bool remove_duplicates: whether or not duplicated element should be removed from output list
    :return: Adds given variable into a list if the variable is not already a list. If the variable is None, an empty
        list is returned. If the variable is a tuple, the tuple is converted into a list
    :rtype: list(object)
    """

    if var is None:
        return list()

    if type(var) is not list:
        if type(var) in [tuple]:
            var = list(var)
        else:
            var = [var]

    if remove_duplicates:
        var = list(set(var))

    return var


def get_files(root_folder, pattern='*'):
    """
    Returns files found in given folder and sub folders taking into account the given pattern
    :param str root_folder: root folder where we want to start searching from
    :param str pattern: find pattern that we use to filter our search for specific file names or extensions
    :return: List of files located in given root folder hierarchy and with the given pattern
    :rtype: list(str)
    """

    if not root_folder or not os.path.isdir(root_folder):
        return list()

    files_found = list()

    for dir_path, dir_names, file_names in os.walk(root_folder):
        for file_name in fnmatch.filter(file_names, pattern):
            files_found.append(clean_path(os.path.join(dir_path, file_name)))

    return list(set(files_found))


def open_folder(path):
    """
    Open folder using OS default settings
    :param path: str, folder path we want to open
    """

    if sys.platform.startswith('darwin'):
        subprocess.Popen(["open", path])
    elif os.name == 'nt':
        os.startfile(path)
    elif os.name == 'posix':
        subprocess.Popen(["xdg-open", path])
    else:
        raise NotImplementedError('OS not supported: {}'.format(os.name))


def open_file(file_path):
    """
    Open file using OS default settings
    :param file_path: str, file path we want to open
    """

    if sys.platform.startswith('darwin'):
        subprocess.call(('open', file_path))
    elif os.name == 'nt':
        os.startfile(file_path)
    elif os.name == 'posix':
        subprocess.call(('xdg-open', file_path))
    else:
        raise NotImplementedError('OS not supported: {}'.format(os.name))


def get_permission(filepath):
    """
    Returns the current permission level
    :param filepath: str
    """

    if os.access(filepath, os.R_OK | os.W_OK | os.X_OK):
        return True

    try:
        os.chmod(filepath, 0o775)
        return True
    except Exception:
        return False


def delete_file(file_path):
    """
    Delete the file by name in the directory
    :param name: str, name of the file to delete
    :param directory: str, the directory where the file is stored
    :return: str, file path that was deleted
    """

    if not os.path.isfile(file_path):
        logger.warning('File "{}" was not deleted.'.format(file_path))
        return False

    try:
        get_permission(file_path)
    except Exception:
        pass

    try:
        os.remove(file_path)
    except Exception as exc:
        pass

    if os.path.isfile(file_path):
        return False

    return True


def delete_folder(folder_name, directory=None):
    """
    Deletes the folder by name in the given directory
    :param folder_name: str, name of the folder to delete
    :param directory: str, the directory path where the folder is stored
    :return: str, folder that was deleted with path
    """

    def delete_read_only_error(action, name, exc):
        """
        Helper to delete read only files
        """

        os.chmod(name, 0o777)
        action(name)

    full_path = folder_name
    if directory:
        full_path = clean_path(os.path.join(directory, folder_name))
    if not os.path.isdir(full_path):
        return None

    try:
        shutil.rmtree(full_path, onerror=delete_read_only_error)
    except Exception as exc:
        logger.warning('Could not remove folder "{}" | {}'.format(full_path, exc))

    return full_path


def debug_object_string(obj, msg):
    """
    Returns a debug string depending of the type of the object

    :param object obj: Python object
    :param str msg: message to log
    :return: debug string
    :rtype: str
    """

    if inspect.ismodule(obj):
        return '[%s module] :: %s' % (obj.__name__, msg)
    elif inspect.isclass(obj):
        return '[%s.%s class] :: %s' % (obj.__module__, obj.__name__, msg)
    elif inspect.ismethod(obj):
        return '[%s.%s.%s method] :: %s' % (obj.im_class.__module__, obj.im_class.__name__, obj.__name__, msg)
    elif inspect.isfunction(obj):
        return '[%s.%s function] :: %s' % (obj.__module__, obj.__name__, msg)


def import_module(module_path, name=None):
    """
    Imports the given module path. If the given module path is a dotted one, import lib will be used. Otherwise, it's
    expected that given module path is the absolute path to the source file. If name argument is not given, then the
    basename without the extension will be used
    :param module_path: str, module path. Can be a dotted path (tpDcc.libs.python.modules) or an absolute one
    :param name: str, name for the imported module which will be used if the module path is an absolute path
    :return: ModuleObject, imported module object
    """

    if is_dotted_module_path(module_path) and not os.path.exists(module_path):
        try:
            return importlib.import_module(module_path)
        except ImportError as exc:
            logger.exception('Failed to load module: "{}" | {}'.format(module_path, exc))
            return None

    try:
        if os.path.exists(module_path):
            if not name:
                name = os.path.splitext(os.path.basename(module_path))[0]
            if name in sys.modules:
                return sys.modules[name]
        if os.path.isdir(module_path):
            module_path = clean_path(os.path.join(module_path, '__init__.py'))
            if not os.path.exists(module_path):
                raise ValueError('Cannot find module path: "{}"'.format(module_path))
        return imp.load_source(name, os.path.realpath(module_path))
    except ImportError:
        logger.error('Failed to load module: "{}"'.format(module_path))
        return None


def iterate_modules(path, exclude=None):
    """
    Iterates all Python modules of the given path
    :param str path: folder path to iterate searching for Python modules
    :param list(str) exclude: list of files to exclude during the searching process
    :return: Iterator with all the found modules
    :rtype: iterator
    """

    if not exclude:
        exclude = list()

    _exclude = ['__init__.py', '__init__.pyc']
    for root, dirs, files in os.walk(path):
        if '__init__.py' not in files:
            continue
        for f in files:
            base_name = os.path.splitext(f)[0]
            if f not in _exclude and base_name not in exclude:
                module_path = clean_path(os.path.join(root, f))
                if f.endswith('.py') or f.endswith('.pyc'):
                    yield module_path


def iterate_module_members(module_to_iterate, predicate=None):
    """
    Iterates all the members of the given modules
    :param ModuleObject module_to_iterate: module object to iterate members of
    :param inspect.cass predicate: if given members will be restricted to given inspect class
    :return:
    :rtype: iterator
    """

    for mod in inspect.getmembers(module_to_iterate, predicate=predicate):
        yield mod


def is_dotted_module_path(module_path):
    """
    Returns whether given module path is a dotted one (tpDcc.libs.python.modules) or not
    :param str module_path:
    :return:
    :rtype: bool
    """

    return len(module_path.split('.')) >= 2


def convert_module_path_to_dotted_path(path):
    """
    Returns a dotted path relative to the given path
    :param str  path: str, Path to module we want to convert to dotted path (eg. myPlugin/folder/test.py)
    :return: dotted path (eg. folder.test)
    :rtype: str
    """

    directory, file_path = os.path.split(path)
    directory = clean_path(directory)
    file_name = os.path.splitext(file_path)[0]
    package_path = [file_name]
    sys_path = [clean_path(p) for p in sys.path]
    drive_letter = os.path.splitdrive(path)[0] + '\\'
    while directory not in sys_path:
        directory, name = os.path.split(directory)
        directory = clean_path(directory)
        if directory == drive_letter or name == '':
            return ''
        package_path.append(name)

    return '.'.join(reversed(package_path))


def read_json(filename):
    """
    Get data from JSON file
    """

    if os.stat(filename).st_size == 0:
        return None
    else:
        try:
            with open(filename, 'r') as json_file:
                data = json.load(json_file)
        except Exception as err:
            logger.warning('Could not read {0}'.format(filename))
            raise err

    return data


def get_percent(value, minimum, maximum):
    if minimum == maximum:
        return 100
    return max(0, min(100, (value - minimum) * 100 / (maximum - minimum)))


def timestamp(f):
    """
    Function decorator that gets the elapsed time with a more descriptive output

    :param f: fn, function
    """

    @wraps(f)
    def wrapper(*args, **kwargs):
        start_time = time.time()
        res = f(*args, **kwargs)
        logger.warning('<{}> Elapsed time : {}'.format(f.func_name, time.time() - start_time))
        return res
    return wrapper


def abstract(fn):
    """
    Decorator that indicates that decorated function should be override
    """

    @wraps(fn)
    def wrapper(*args, **kwargs):
        msg = 'DCC Abstract function {} has not been overridden.'.format(fn)
        raise NotImplementedError(debug_object_string(fn, msg))

    return wrapper


class Singleton(object):
    """
    Implements Singleton pattern design as a class decorator in Python
    """

    all_instances = list()

    @staticmethod
    def destroy_all():
        for instance in Singleton.all_instances:
            instance.destroy()

    def __init__(self, cls):
        self.cls = cls
        self.instance = None
        self.all_instances.append(self)

    def destroy(self):
        del self.instance
        self.instance = None

    def __call__(self, *args, **kwargs):
        if self.instance is None:
            self.instance = self.cls(*args, **kwargs)

        return self.instance



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains DCC abstract UI implementation
"""

from __future__ import print_function, division, absolute_import

from artella.core.dcc import reroute
from artella.core.utils import abstract


@reroute
@abstract
def get_main_window():
    """
    Returns Qt object that references to the main DCC window we are working on

    :return: An instance of the current DCC Qt main window
    :rtype: QMainWindow or QWidget or None
    """

    pass


@reroute
@abstract
def show_info(title, message):
    """
    Shows an information dialog that users need to accept/reject.

    :param str title: text that is displayed in the title bar of the dialog
    :param str message: text which is shown to the user telling them what operation they need to confirm
    :return: True if the user accepted the operation; False otherwise.
    :rtype: bool
    """

    pass


@reroute
@abstract
def show_question(title, message, cancel=True):
    """
    Shows a question message box that can be used to show question text to users.

    :param str title: text that is displayed in the title bar of the dialog
    :param str message: text which is shown to the user telling them what operation they need to confirm
    :param bool cancel: Whether or not cancel button should appear in question message box
    :return: True if the user presses the Ok button; False if the user presses the No button; None if the user preses
        the Cancel button
    :rtype: bool or None
    """

    pass


@reroute
@abstract
def show_warning(title, message, print_message=False):
    """
    Shows a warning message box that can be used to show warning text to users.

    :param str title: text that is displayed in the title bar of the dialog
    :param str message: default text which is placed n the plain text edit
    :param bool print_message: whether or not print message in DCC output command window
    """

    pass


@reroute
@abstract
def show_error(title, message, print_message=False):
    """
    Shows an error message box that can be used to show critical text to users.

    :param str title: text that is displayed in the title bar of the dialog
    :param str message: default text which is placed n the plain text edit
    :param bool print_message: whether or not print message in DCC output command window
    """

    pass


@reroute
@abstract
def input_comment(title, label, text=''):
    """
    Shows a comment input dialog that users can use to input text.

    :param str title: text that is displayed in the title bar of the dialog
    :param str label: text which is shown to the user telling them what kind of text they need to input
    :param str text: default text which is placed in the comment section
    :return: Tuple containing as the first element the text typed by the user and as the second argument a boolean
        that will be True if the user clicked on the Ok button or False if the user cancelled the operation
    :rtype: tuple(str, bool)
    """

    pass



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains Artella Plugins framework implementation
"""

from __future__ import print_function, division, absolute_import

import os
import sys
import logging
import inspect

import artella
import artella.dcc as dcc
import artella.register as register
from artella.core import consts, utils

logger = logging.getLogger('artella')


class ArtellaPlugin(object):

    ID = ''                 # Unique ID of the Artella Plugin
    INDEX = -1              # Index of the plugin. This index will control the instantiation order
    VERSION = None          # Version of the Artella Plugin
    PACKAGE = None          # Package Artella Plugin belongs to

    def __init__(self, config_dict=None, manager=None):

        self._config_dict = config_dict or dict()
        self._manager = manager
        self._stats = ArtellaPluginStats(self)
        self._loaded = False

        self.init()

    def is_loaded(self):
        """
        Returns whether or not this plugin is loaded
        :return: True if the plugin is loaded; False otherwise.
        :rtype: bool
        """

        return self._loaded

    def init(self):
        """
        Function that is called when plugin is instantiated
        """

        if self._loaded:
            dcc.execute_deferred(self.cleanup)

        dcc.execute_deferred(self.init_ui)

        self._loaded = True

    def init_ui(self):
        """
        Function that initializes plugin UI related functionality
        """

        main_menu = artella.DccPlugin().get_main_menu()
        if not main_menu:
            return

        plugin_menu = self._config_dict.get('menu')
        plugin_package = self._config_dict.get('package', 'Artella')
        plugin_dccs = self._config_dict.get('dcc', list())

        can_load_plugin = True
        if plugin_dccs:
            can_load_plugin = dcc.name() in plugin_dccs

        if can_load_plugin:
            if plugin_menu and 'label' in plugin_menu:
                menu_parents = plugin_menu.get('parents', None)
                if menu_parents:
                    current_parent = 'Artella' if dcc.check_menu_exists('Artella') else None
                    for menu_parent in menu_parents:
                        if not dcc.check_menu_exists(menu_parent):

                            # TODO: Before More Artella menu addition we force the creation of a
                            # TODO: separator. We should find a way to avoid hardcoded this.
                            icon = ''
                            if menu_parent == 'More Artella':
                                dcc.add_menu_separator('Artella')
                                icon = 'artella.png'

                            dcc.add_sub_menu_item(menu_parent, parent_menu=current_parent, icon=icon)
                        current_parent = menu_parent

                menu_label = plugin_menu['label']
                menu_command = plugin_menu['command']
                menu_icon = self._config_dict.get('icon', '')
                if menu_parents:
                    menu_parent = menu_parents[-1]
                    dcc.add_menu_item(menu_label, menu_command, menu_parent, icon=menu_icon)
                else:
                    dcc.add_menu_item(menu_label, menu_command, main_menu, icon=menu_icon)

    def cleanup(self):
        """
        Function that is called when the plugin is disabled
        """

        plugin_menu = self._config_dict.get('menu')
        menu = dcc.get_menu('Artella')

        if plugin_menu and menu:
            menu_label = plugin_menu['label']
            dcc.remove_menu_item(menu_label, menu)

        self._loaded = False

    @property
    def manager(self):
        """
        Returns Artella Plugins manager instance that is owner of this plugin

        :return: Artella Plugin manager instance that owns this plugin
        :rtype: ArtellaPluginsManager
        """

        return self._manager

    @property
    def stats(self):
        """
        Returns Artella Plugin Stats instance that stores useful data related with the plugin

        :return: Artella Plugin Stats instance with useful data of the plugin
        :rtype: ArtellaPluginStats
        """

        return self._stats


class ArtellaPluginStats(object):
    def __init__(self, plugin):
        self._plugin = plugin
        self._id = self._plugin.ID
        self._start_time = 0.0
        self._end_time = 0.0
        self._execution_time = 0.0
        self._info = dict()
        self._init()

    @property
    def start_time(self):
        """
        Returns the time the plugin was executed

        :return: Time the plugin was executed
        :rtype: str
        """

        return self._start_time

    @start_time.setter
    def start_time(self, value):
        """
        Sets the time the plugin was executed

        :param str value: Time the plugin was executed
        """

        self._start_time = value

    @property
    def end_time(self):
        """
        Returns the time the plugin ended its execution

        :return: Time the plugin was executed
        :rtype: str
        """

        return self._end_time

    @end_time.setter
    def end_time(self, value):
        """
        Sets the time the plugin ended its execution

        :param str value: Time the plugin ended its execution
        """

        self._end_time = value

    @property
    def execution_time(self):
        """
        Returns the the total amount of execution time of the plugin

        :return: Total execution time of the plugin during current session in seconds
        :rtype: float
        """

        return self._execution_time

    def _init(self):
        """
        Internal function that initializes info for the plugin and its environment
        """

        from artella import dcc

        self._info.update({
            'name': self._plugin.__class__.__name__,
            'module': self._plugin.__class__.__module__,
            'filepath': inspect.getfile(self._plugin.__class__),
            'id': self._id,
            'application': dcc.name()
        })


class ArtellaPluginsManager(object):

    PLUGIN_INTERFACE = ArtellaPlugin

    def __init__(self):
        self._plugins = dict()
        self._plugin_paths = list()

    @property
    def plugins(self):
        """
        Returns a dictionary containing all the info of already registered Artella plugins
        :return: Dictionary with already loaded Artella plugins information
        :rtype :dict
        """

        return self._plugins

    @property
    def plugin_paths(self):
        """
        Returns a list of paths where Artella Plugin Manager will find for plugins
        :return: List of plugin directory paths
        :rtype: list(str)
        """

        return self._plugin_paths

    def get_plugin_by_name(self, plugin_name):
        """
        Returns plugin instance by its name
        :param plugin_name: Name of the plugin to retrieve
        :return: Returns plugin instance if a plugin with the given name is loaded; None otherwise.
        :rtype: ArtellaPlugin
        """

        if not self._plugins:
            return None

        for plugin_id, plugin_dict in self._plugins.items():
            plug_name = plugin_dict['name']
            if plugin_name == plug_name:
                return plugin_dict['plugin_instance']

        return None

    def get_plugin_by_id(self, plugin_id):
        """
        Returns plugin instance by its ID
        :param plugin_id: Name of the plugin to retrieve (for example, artella-plugin-test)
        :return: Returns plugin instance if a plugin with the given ID is loaded; None otherwise.
        :rtype: ArtellaPlugin
        """

        if not self._plugins:
            return None

        for plug_id, plugin_dict in self._plugins.items():
            if plug_id == plugin_id:
                return plugin_dict['plugin_instance']

    def register_paths(self, plugin_paths):
        """
        Registers given path into the list of Artella plugin paths.
        :param str plugin_paths: Path that will be used by Artella Plugins Manager to search plugins into.
        """

        plugin_paths = utils.force_list(plugin_paths, remove_duplicates=True)

        for plugin_path in plugin_paths:
            plugin_path = utils.clean_path(plugin_path)
            if not plugin_path or not os.path.isdir(plugin_path) or plugin_path in self._plugin_paths:
                return

            self._plugin_paths.append(plugin_path)

    def load_registered_plugins(self):
        """
        Loads all the plugins found in the registered plugin paths
        :return:
        """

        plugin_paths = list(set([
            utils.clean_path(plugin_path) for plugin_path in self._plugin_paths if os.path.isdir(plugin_path)]))
        if not plugin_paths:
            logger.info('No Artella Plugins found to load!')
            return

        found_paths = dict()

        for plugin_path in plugin_paths:
            for root, dirs, files in os.walk(plugin_path):
                if consts.ARTELLA_PLUGIN_CONFIG not in files:
                    continue
                clean_path = utils.clean_path(root)
                found_paths[clean_path] = os.path.join(root, consts.ARTELLA_PLUGIN_CONFIG)

        if not found_paths:
            logger.info('No plugins found in registered plugin paths: {}'.format(self._plugin_paths))
            return

        for plugin_path in plugin_paths:
            for plugin_dir in os.listdir(plugin_path):
                clean_path = utils.clean_path(os.path.join(plugin_path, plugin_dir))
                if os.path.isdir(clean_path) and clean_path not in sys.path:
                    sys.path.append(clean_path)

        for plugin_path, plugin_config in found_paths.items():

            # Search sub modules paths
            sub_modules_found = list()
            for sub_module in utils.iterate_modules(plugin_path):
                file_name = os.path.splitext(os.path.basename(sub_module))[0]
                if file_name.startswith('_') or file_name.startswith('test_') or sub_module.endswith('.pyc'):
                    continue
                if not sub_module or sub_module in sub_modules_found:
                    continue
                sub_modules_found.append(sub_module)
            if not sub_modules_found:
                continue

            # Find specific DCC plugin implementation
            sub_module = None
            dcc_name = dcc.name()
            for sub_module_found in sub_modules_found:
                parent_dir_name = os.path.basename(os.path.dirname(sub_module_found))
                if parent_dir_name == dcc_name:
                    sub_module = sub_module_found
                    break
            if not sub_module:
                sub_module = sub_modules_found[0]

            module_path = utils.convert_module_path_to_dotted_path(os.path.normpath(sub_module))

            try:
                sub_module_obj = utils.import_module(module_path)
            except Exception as exc:
                logger.error('Error while importing Artella Plugin module: {} | {}'.format(module_path, exc))
                continue
            if not sub_module_obj:
                continue

            plugin_version = None
            module_path_dir = module_path.rsplit('.', 1)[0]
            version_module_path = '{}.__version__'.format(module_path_dir)
            version_module_path = version_module_path.replace('.{}.__'.format(dcc_name), '.__')
            try:
                version_module_obj = utils.import_module(version_module_path)
            except Exception:
                version_module_obj = None
            if version_module_obj:
                try:
                    plugin_version = version_module_obj.get_version()
                except Exception as exc:
                    logger.warning(
                        'Impossible to retrieve version for Artella Plugin module: {} | {}'.format(module_path, exc))

            for member in utils.iterate_module_members(sub_module_obj, predicate=inspect.isclass):
                self.register_plugin(member[1], plugin_config, os.path.dirname(sub_module), plugin_version)

        if not self._plugins:
            logger.warning('No Artella plugins found to load!')
            return

        ordered_plugins_list = list()
        for plugin_id, plugin_dict in self._plugins.items():
            plugin_class = plugin_dict['class']
            plugin_index = plugin_class.INDEX or -1
            index = 0
            for i, plugin_item in enumerate(ordered_plugins_list):
                plugin_item_index = plugin_item.values()[0]['index']
                if plugin_index < plugin_item_index:
                    index += 1
            ordered_plugins_list.insert(index, {plugin_id: {'index': plugin_index, 'dict': plugin_dict}})

        for plugin_item in ordered_plugins_list:
            plugin_id = plugin_item.keys()[0]
            plugin_dict = plugin_item.values()[0]['dict']
            plugin_class = plugin_dict['class']
            plugin_config_dict = plugin_dict.get('config', dict())
            try:
                plugin_inst = plugin_class(plugin_config_dict, manager=self)
            except Exception as exc:
                logger.error('Impossible to instantiate Artella Plugin: "{}" | {}'.format(plugin_id, exc))
                continue
            self._plugins[plugin_id]['plugin_instance'] = plugin_inst

    def register_plugin(self, class_obj, config_path, plugin_path, plugin_version=None):
        """
        Registers an Artella plugin instance into the manager
        :param class_obj:
        :param config_path:
        :param plugin_path:
        :param plugin_version:
        :return:
        :rtype: bool
        """

        if not config_path or not os.path.isfile(config_path):
            logger.warning(
                'Impossible to register Artella Plugin: {} because its config file does not exists: {}!'.format(
                    class_obj, config_path))
            return True

        if not issubclass(class_obj, self.PLUGIN_INTERFACE):
            return True

        plugin_id = None
        if hasattr(class_obj, 'ID'):
            plugin_id = getattr(class_obj, 'ID')
        if not plugin_id:
            plugin_id = class_obj.__name__
        if plugin_id in self._plugins:
            logger.warning('Artella Plugin: "{}" already registered!'.format(plugin_id))
            return True

        try:
            plugin_config = utils.read_json(config_path)
        except Exception as exc:
            logger.warning(
                'Artella Plugin "{}" configuration file "{}" has not a proper structure. Skipping plugin ...'.format(
                    plugin_id, config_path))
            return True

        plugin_name = plugin_config.get('name', None)
        plugin_package = plugin_config.get('package', None)
        plugin_icon = plugin_config.get('icon', None)
        plugin_resources = plugin_config.get('resources', None)

        # Register DCC resources, both DCC specific plugin implementation
        # resources and generic implementation resources
        plugin_resources_paths = list()
        if plugin_resources and plugin_path:
            plugin_path_base = os.path.basename(plugin_path)
            if plugin_path_base == dcc.name():
                plugin_resources_paths.append(os.path.join(plugin_path, plugin_resources))
                plugin_resources_paths.append(os.path.join(os.path.dirname(plugin_path), plugin_resources))
            else:
                plugin_resources_paths.append(os.path.join(plugin_path, plugin_resources))
            for plugin_resources_path in plugin_resources_paths:
                if os.path.isdir(plugin_resources_path):
                    dcc.register_dcc_resource_path(plugin_resources_path)

        self._plugins[plugin_id] = {
            'name': plugin_name,
            'package': plugin_package,
            'icon': plugin_icon,
            'class': class_obj,
            'config': plugin_config,
            'version': plugin_version,
            'resource_paths': plugin_resources_paths
        }

        logger.info('Artella Plugin: "{}" registered successfully!'.format(plugin_id))

        return True

    def shutdown(self, dev=False):
        """
        Unloads all current loaded Artella plugins
        """

        if not self._plugins:
            return

        for plugin_id, plugin_dict in self._plugins.items():
            plugin_inst = plugin_dict.get('plugin_instance', None)
            if not plugin_inst:
                continue
            if dev:
                plugin_inst.cleanup()
            else:
                dcc.execute_deferred(plugin_inst.cleanup)


@utils.Singleton
class ArtellaPluginsManagerSingleton(ArtellaPluginsManager, object):
    def __init__(self):
        ArtellaPluginsManager.__init__(self)


register.register_class('PluginsMgr', ArtellaPluginsManagerSingleton)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains Artella Splash widget
"""

from __future__ import print_function, division, absolute_import

import logging

import artella
from artella.core import qtutils, utils

if qtutils.QT_AVAILABLE:
    from artella.externals.Qt import QtCore, QtWidgets, QtGui

logger = logging.getLogger('artella')


if not qtutils.QT_AVAILABLE:
    class SplashScreen(object):
        def __init__(self, *args, **kwargs):
            pass

    class ProgressCricle(object):
        def __init__(self, *args, **kwargs):
            pass
else:
    class SplashScreen(QtWidgets.QSplashScreen, object):
        def __init__(self, *args, **kwargs):
            super(SplashScreen, self).__init__(*args, **kwargs)

        def mousePressEvent(self, event):
            pass

    class ProgressCricle(QtWidgets.QProgressBar, object):
        def __init__(self, parent=None, **kwargs):
            super(ProgressCricle, self).__init__(parent)

            self._main_layout = QtWidgets.QHBoxLayout()
            self._default_label = QtWidgets.QLabel()
            self._default_label.setAlignment(QtCore.Qt.AlignCenter)
            self._main_layout.addWidget(self._default_label)
            self.setLayout(self._main_layout)
            self._color = QtGui.QColor(221, 235, 230)
            self._width = kwargs.get('width', 140)

            self.setTextDirection(self.Direction.BottomToTop)

            self._start_angle = 90 * 16
            self._max_delta_angle = 360 * 16
            self._height_factor = 1.0
            self._width_factor = 1.0

            self.setFixedSize(QtCore.QSize(self._width * self._width_factor, self._width * self._height_factor))

        def set_widget(self, widget):
            self.setTextVisible(False)
            self._main_layout.addWidget(widget)

        def paintEvent(self, event):
            if self.text() != self._default_label.text():
                self._default_label.setText(self.text())
            if self.isTextVisible() != self._default_label.isVisible():
                self._default_label.setVisible(self.isTextVisible())

            percent = utils.get_percent(self.value(), self.minimum(), self.maximum())
            total_width = self._width
            pen_width = int(3 * total_width / 50.0)
            radius = total_width - pen_width - 1

            painter = QtGui.QPainter(self)
            painter.setRenderHints(QtGui.QPainter.Antialiasing)

            # draw background circle
            pen_background = QtGui.QPen()
            pen_background.setWidth(pen_width)
            pen_background.setColor(QtGui.QColor(80, 120, 110))
            pen_background.setCapStyle(QtCore.Qt.RoundCap)
            painter.setPen(pen_background)
            painter.drawArc(pen_width / 2.0 + 1,
                            pen_width / 2.0 + 1,
                            radius,
                            radius,
                            self._start_angle,
                            -self._max_delta_angle)

            # draw foreground circle
            pen_foreground = QtGui.QPen()
            pen_foreground.setWidth(pen_width)
            pen_foreground.setColor(self._color)
            pen_foreground.setCapStyle(QtCore.Qt.RoundCap)
            painter.setPen(pen_foreground)
            painter.drawArc(
                pen_width / 2.0 + 1, pen_width / 2.0 + 1, radius, radius,
                self._start_angle, -percent * 0.01 * self._max_delta_angle)
            painter.end()


class SplashDialog(artella.Dialog, object):
    def __init__(self, parent=None, **kwargs):
        super(SplashDialog, self).__init__(parent, use_artella_header=False, **kwargs)

    def get_main_layout(self):
        main_layout = QtWidgets.QVBoxLayout()
        main_layout.setContentsMargins(10, 10, 10, 10)
        main_layout.setSpacing(5)

        return main_layout

    def setup_ui(self):
        super(SplashDialog, self).setup_ui()

        self.setWindowFlags(QtCore.Qt.Window | QtCore.Qt.FramelessWindowHint | QtCore.Qt.WA_DeleteOnClose)
        self.setAttribute(QtCore.Qt.WA_TranslucentBackground)

        splash_pixmap = artella.ResourcesMgr().pixmap('artella_splash')
        splash = SplashScreen(splash_pixmap)
        splash.setMask(splash_pixmap.mask())
        self._splash_layout = QtWidgets.QVBoxLayout()
        self._splash_layout.setAlignment(QtCore.Qt.AlignBottom)
        splash.setLayout(self._splash_layout)
        self.main_layout.addWidget(splash)

        size_width = splash_pixmap.size().width() + 20
        size_height = splash_pixmap.size().height() + 20
        self.setFixedSize(QtCore.QSize(size_width, size_height))

        shadow_effect = QtWidgets.QGraphicsDropShadowEffect(self)
        shadow_effect.setBlurRadius(qtutils.dpi_scale(15))
        shadow_effect.setColor(QtGui.QColor(0, 0, 0, 150))
        shadow_effect.setOffset(qtutils.dpi_scale(0))
        self.setGraphicsEffect(shadow_effect)


class InfoSplashDialog(SplashDialog, object):
    def __init__(self, parent=None, **kwargs):
        super(InfoSplashDialog, self).__init__(parent, **kwargs)

    def setup_ui(self):
        super(InfoSplashDialog, self).setup_ui()

        self._progress_text = QtWidgets.QLabel('Wait please ...')
        progress_text_layout = QtWidgets.QHBoxLayout()
        progress_text_layout.addStretch()
        progress_text_layout.addWidget(self._progress_text)
        progress_text_layout.addStretch()

        self._splash_layout.addLayout(progress_text_layout)

    def set_text(self, text):
        self._progress_text.setText(text)


class ProgressSplashDialog(SplashDialog, object):
    def __init__(self, parent=None, **kwargs):

        self._is_cancelled = False

        super(ProgressSplashDialog, self).__init__(parent, **kwargs)

    def keyPressEvent(self, event):
        if event.key() == QtCore.Qt.Key_Escape:
            self._is_cancelled = True
        super(ProgressSplashDialog, self).keyPressEvent(event)

    def setup_ui(self):
        super(ProgressSplashDialog, self).setup_ui()

        self._progress = ProgressCricle()
        progress_lyt = QtWidgets.QHBoxLayout()
        progress_lyt.addStretch()
        progress_lyt.addWidget(self._progress)
        progress_lyt.addStretch()

        self._progress_text = QtWidgets.QLabel('Wait please ...')
        progress_txt_lyt = QtWidgets.QHBoxLayout()
        progress_txt_lyt.addStretch()
        progress_txt_lyt.addWidget(self._progress_text)
        progress_txt_lyt.addStretch()

        self._splash_layout.addLayout(progress_lyt)
        self._splash_layout.addLayout(progress_txt_lyt)

    def set_progress_text(self, text):
        self._progress.setFormat(text)

    def start(self):

        from artella import dcc

        self._progress.setValue(0)
        self._progress_text.setText('')

        if dcc.is_batch():
            self._log_progress()
            return

        self.show()

    def end(self):
        from artella import dcc

        self._log_progress()

        if dcc.is_batch():
            return

        self.fade_close()

    def get_max_progress_value(self):
        return self._progress.maximum()

    def set_max_progress_value(self, max_value):
        self._progress.setMaximum(max_value)

    def get_progress_value(self):
        return self._progress.value()

    def set_progress_value(self, value, status=''):
        self._progress.setValue(value)
        self._progress_text.setText(str(status))

    def is_cancelled(self):
        return self._is_cancelled

    def _log_progress(self):
        """
         Internal function that logs current progress into DCC output window
         """

        logger.debug('{} - {}'.format(self._progress_text.text(), self._progress.value()))
